@@ -1,5 +1,5 @@
 /* cardglue.c - mainly dispatcher for card related functions.
- * Copyright (C) 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -190,6 +190,37 @@ send_status_info (ctrl_t ctrl, const char *keyword, ...)
   va_end (arg_ptr);
 }
 
+/* Send a ready formatted status line via assuan.  */
+void
+send_status_direct (ctrl_t ctrl, const char *keyword, const char *args)
+{
+  char buf[950];
+
+  if (strchr (args, '\n'))
+    log_error ("error: LF detected in status line - not sending\n");
+  else
+    {
+      snprintf (buf, sizeof buf, "%s%s%s",
+                keyword, args? " ":"", args? args:"");
+      if (ctrl && ctrl->status_cb)
+        ctrl->status_cb (ctrl->status_cb_arg, buf);
+    }
+}
+
+
+
+void
+gcry_mpi_release (MPI a)
+{
+  mpi_free (a);
+}
+
+MPI 
+gcry_mpi_set_opaque (MPI a, void *p, unsigned int len)
+{
+  return mpi_set_opaque (a, p, len);
+}
+
 
 /* Replacement function of the Libgcrypt onewhich is used in gnupg
    1.9.  Thus function computes the digest of ALGO from the data in
@@ -208,6 +239,17 @@ gcry_md_hash_buffer (int algo, void *digest,
 }
 
 
+/* This function simply returns the name of the algorithm or some
+   constant string when there is no algo.  It will never return
+   NULL.  */
+const char *
+gcry_md_algo_name (int algorithm)
+{
+  const char *s = digest_algo_to_string (algorithm);
+  return s ? s : "?";
+}
+
+
 /* This is a limited version of the one in 1.9 but it should be
    sufficient here. */
 void
@@ -297,6 +339,7 @@ agent_release_card_info (struct agent_card_info_s *info)
     return;
 
   xfree (info->serialno); info->serialno = NULL;
+  xfree (info->apptype); info->apptype = NULL;
   xfree (info->disp_name); info->disp_name = NULL;
   xfree (info->disp_lang); info->disp_lang = NULL;
   xfree (info->pubkey_url); info->pubkey_url = NULL;
@@ -339,7 +382,7 @@ open_card_via_agent (int *scd_available)
   if (!ctx)
     return NULL;
 
-  /* Request the serialbnumber of the card.  If we get
+  /* Request the serialnumber of the card.  If we get
      NOT_SUPPORTED or NO_SCDAEMON back, the gpg-agent either has
      disabled scdaemon or it can't be used.  We close the connection
      in this case and use our own code.  This may happen if just the
@@ -395,7 +438,7 @@ open_card (void)
       if (app)
         goto ready; /* Yes, there is a agent with a usable card, go that way. */
       if (scd_available)
-        return NULL; /* agent avilabale but card problem. */
+        return NULL; /* Agent available but card problem. */
     }
 
 
@@ -448,7 +491,7 @@ open_card (void)
     }
 
  ready:
-  app->initialized = 1;
+  app->ref_count = 1;
   current_app = app;
   if (is_status_enabled () )
     {
@@ -629,6 +672,15 @@ store_serialno (const char *line)
   return p;
 }
 
+/* Return a new malloced string by unescaping the string S.  Escaping
+   is percent escaping and '+'/space mapping.  A binary nul will
+   silently be replaced by a 0xFF.  Function returns NULL to indicate
+   an out of memory status. */
+static char *
+unescape_status_string (const unsigned char *s)
+{
+  return unescape_percent_string (s);
+}
 
 
 static assuan_error_t
@@ -649,6 +701,13 @@ learn_status_cb (void *opaque, const char *line)
     {
       xfree (parm->serialno);
       parm->serialno = store_serialno (line);
+      parm->is_v2 = (strlen (parm->serialno) >= 16 
+                     && xtoi_2 (parm->serialno+12) >= 2 );
+    }
+  else if (keywordlen == 7 && !memcmp (keyword, "APPTYPE", keywordlen))
+    {
+      xfree (parm->apptype);
+      parm->apptype = unescape_status_string (line);
     }
   else if (keywordlen == 9 && !memcmp (keyword, "DISP-NAME", keywordlen))
     {
@@ -711,6 +770,30 @@ learn_status_cb (void *opaque, const char *line)
           xfree (buf);
         }
     }
+  else if (keywordlen == 6 && !memcmp (keyword, "EXTCAP", keywordlen))
+    {
+      char *p, *p2, *buf;
+      int abool;
+
+      buf = p = unescape_status_string (line);
+      if (buf)
+        {
+          for (p = strtok (buf, " "); p; p = strtok (NULL, " "))
+            {
+              p2 = strchr (p, '=');
+              if (p2)
+                {
+                  *p2++ = 0;
+                  abool = (*p2 == '1');
+                  if (!strcmp (p, "ki"))
+                    parm->extcap.ki = abool;
+                  else if (!strcmp (p, "aac"))
+                    parm->extcap.aac = abool;
+                }
+            }
+          xfree (buf);
+        }
+    }
   else if (keywordlen == 7 && !memcmp (keyword, "KEY-FPR", keywordlen))
     {
       int no = atoi (line);
@@ -761,6 +844,18 @@ learn_status_cb (void *opaque, const char *line)
       xfree (parm->private_do[no]);
       parm->private_do[no] = unescape_percent_string (line);
     }
+  else if (keywordlen == 8 && !memcmp (keyword, "KEY-ATTR", keywordlen))
+    {
+      int keyno, algo, nbits;
+
+      sscanf (line, "%d %d %d", &keyno, &algo, &nbits);
+      keyno--;
+      if (keyno >= 0 && keyno < DIM (parm->key_attr))
+        {
+          parm->key_attr[keyno].algo = algo;
+          parm->key_attr[keyno].nbits = nbits;
+        }
+    }
  
   return 0;
 }
@@ -801,10 +896,13 @@ agent_learn (struct agent_card_info_s *info)
           send_status_info (&ctrl, "SERIALNO",
                             serial, strlen(serial), NULL, 0);
           xfree (serial);
-          rc = app->fnc.learn_status (app, &ctrl);
+          rc = app->fnc.learn_status (app, &ctrl, 0);
         }
     }
 
+  if (!rc)
+    agent_scd_getattr ("KEY-ATTR", info);
+
   return rc;
 }
 
@@ -1132,7 +1230,7 @@ genkey_status_cb (void *opaque, const char *line)
 /* Send a GENKEY command to the SCdaemon. */
 int 
 agent_scd_genkey (struct agent_card_genkey_s *info, int keyno, int force,
-                  const char *serialno)
+                  const char *serialno, u32 *createtime)
 {
   app_t app;
   char line[ASSUAN_LINELENGTH];
@@ -1166,6 +1264,7 @@ agent_scd_genkey (struct agent_card_genkey_s *info, int keyno, int force,
       ctrl.status_cb_arg = info;
       rc = app->fnc.genkey (app, &ctrl, line,
                             force? 1:0,
+                            *createtime,
                             pin_cb, &parm);
     }
 
@@ -1224,7 +1323,13 @@ agent_scd_pksign (const char *serialno, int hashalgo,
         {
           init_membuf (&data, 1024);
           snprintf (line, DIM(line)-1, "SCD PKSIGN %s%s",
-                    hashalgo == GCRY_MD_RMD160? "--hash=rmd160 ": "",
+                    hashalgo == GCRY_MD_SHA1?   "--hash=sha1 ":
+                    hashalgo == GCRY_MD_SHA224? "--hash=sha224 ":
+                    hashalgo == GCRY_MD_SHA256? "--hash=sha256 ":
+                    hashalgo == GCRY_MD_SHA384? "--hash=sha384 ":
+                    hashalgo == GCRY_MD_SHA512? "--hash=sha512 ":
+                    hashalgo == GCRY_MD_RMD160? "--hash=rmd160 ":
+                    hashalgo == GCRY_MD_MD5?    "--hash=md5 "   : "",
                     serialno);
           line[DIM(line)-1] = 0;
           rc = test_transact (assuan_transact (app->assuan_ctx, line,
@@ -1428,3 +1533,26 @@ agent_clear_pin_cache (const char *sn)
       xfree (cacheid);
     }
 }
+
+
+int 
+agent_scd_writecert (const char *certidstr,
+                     const unsigned char *certdata, size_t certdatalen)
+{
+  /* It does not make sense to implement this rarely used and mainly
+     interactive command in GPG-1.  GPG-2 is better suited for this.  */ 
+  not_in_gpg1_notice ();
+  return gpg_error (GPG_ERR_NOT_SUPPORTED);
+}
+
+
+int 
+agent_scd_readcert (const char *certidstr,
+                    void **r_buf, size_t *r_buflen)
+{
+  /* It does not make sense to implement this rarely used and mainly
+     interactive command in GPG-1.  GPG-2 is better suited for this.  */ 
+  *r_buf = NULL;
+  not_in_gpg1_notice ();
+  return gpg_error (GPG_ERR_NOT_SUPPORTED);
+}
