@@ -1,6 +1,6 @@
 /* iobuf.c  -  file handling
- * Copyright (C) 1998, 1999, 2000, 2001, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2008,
+ *               2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -27,7 +27,7 @@
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h> 
+#include <fcntl.h>
 #include <unistd.h>
 #ifdef HAVE_DOSISH_SYSTEM
 #include <windows.h>
@@ -42,14 +42,24 @@
 #include "dynload.h"
 #include "iobuf.h"
 
-/* The size of the internal buffers. 
+#ifdef __VMS
+# include "vms.h"
+# define open open_vms
+#endif /* def __VMS */
+
+/* The size of the internal buffers.
    NOTE: If you change this value you MUST also adjust the regression
-   test "armored_key_8192" in armor.test! */
+   test "armored_key_8192" and "nopad_armored_msg" in armor.test! */
 #define IOBUF_BUFFER_SIZE  8192
 
 
 #undef FILE_FILTER_USES_STDIO
 
+/* To avoid a potential DoS with compression packets we better limit
+   the number of filters in a chain.  */
+#define MAX_NESTING_FILTER 64
+
+
 #ifdef HAVE_DOSISH_SYSTEM
 #define USE_SETMODE 1
 #endif
@@ -71,8 +81,8 @@ typedef struct {
  } file_filter_ctx_t ;
 #else
 #define my_fileno(a)  (a)
-#define my_fopen_ro(a,b) fd_cache_open ((a),(b)) 
-#define my_fopen(a,b) direct_open ((a),(b)) 
+#define my_fopen_ro(a,b) fd_cache_open ((a),(b))
+#define my_fopen(a,b) direct_open ((a),(b))
 #ifdef HAVE_DOSISH_SYSTEM
 typedef HANDLE FILEP_OR_FD;
 #define INVALID_FP  ((HANDLE)-1)
@@ -94,7 +104,7 @@ typedef struct {
      char fname[1]; /* name of the file */
  } file_filter_ctx_t ;
 
- struct close_cache_s { 
+ struct close_cache_s {
     struct close_cache_s *next;
     FILEP_OR_FD fp;
     char fname[1];
@@ -148,7 +158,7 @@ fd_cache_strcmp (const char *a, const char *b)
 #ifdef HAVE_DOSISH_SYSTEM
   for (; *a && *b; a++, b++)
     {
-      if (*a != *b && !((*a == '/' && *b == '\\') 
+      if (*a != *b && !((*a == '/' && *b == '\\')
                         || (*a == '\\' && *b == '/')) )
         break;
     }
@@ -162,10 +172,11 @@ fd_cache_strcmp (const char *a, const char *b)
  * Invalidate (i.e. close) a cached iobuf or all iobufs if NULL is
  * used for FNAME.
  */
-static void
+static int
 fd_cache_invalidate (const char *fname)
 {
     CLOSE_CACHE cc;
+    int err=0;
 
     if (!fname) {
         if( DBG_IOBUF )
@@ -181,7 +192,7 @@ fd_cache_invalidate (const char *fname)
                 cc->fp = INVALID_FP;
             }
         }
-        return;
+        return err;
     }
 
     if( DBG_IOBUF )
@@ -192,16 +203,43 @@ fd_cache_invalidate (const char *fname)
             if( DBG_IOBUF )
                 log_debug ("                did (%s)\n", cc->fname);
 #ifdef HAVE_DOSISH_SYSTEM
-            CloseHandle (cc->fp);
+            if(CloseHandle (cc->fp)==0)
+	      err=-1;
 #else
-	    close(cc->fp);
+	    err=close(cc->fp);
 #endif
             cc->fp = INVALID_FP;
         }
     }
+
+    return err;
 }
 
+static int
+fd_cache_synchronize(const char *fname)
+{
+  int err=0;
 
+#ifndef HAVE_DOSISH_SYSTEM
+  CLOSE_CACHE cc;
+
+  if( DBG_IOBUF )
+    log_debug ("fd_cache_synchronize (%s)\n", fname);
+
+  for (cc=close_cache; cc; cc = cc->next )
+    {
+      if ( cc->fp != INVALID_FP && !fd_cache_strcmp (cc->fname, fname) )
+	{
+	  if( DBG_IOBUF )
+	    log_debug ("                did (%s)\n", cc->fname);
+
+	  err=fsync(cc->fp);
+	}
+    }
+#endif
+
+  return err;
+}
 
 static FILEP_OR_FD
 direct_open (const char *fname, const char *mode)
@@ -262,7 +300,7 @@ direct_open (const char *fname, const char *mode)
     {
         struct stat buf;
         int rc = stat( fname, &buf );
-        
+
         /* Don't allow iobufs on directories */
         if( !rc && S_ISDIR(buf.st_mode) && !S_ISREG(buf.st_mode) )
             return __set_errno( EISDIR );
@@ -275,7 +313,7 @@ direct_open (const char *fname, const char *mode)
 
 
 /*
- * Instead of closing an FD we keep it open and cache it for later reuse 
+ * Instead of closing an FD we keep it open and cache it for later reuse
  * Note that this caching strategy only works if the process does not chdir.
  */
 static void
@@ -291,7 +329,7 @@ fd_cache_close (const char *fname, FILEP_OR_FD fp)
         close(fp);
 #endif
         if( DBG_IOBUF )
-            log_debug ("fd_cache_close (%p) real\n", (void*)fp);
+          log_debug ("fd_cache_close (%d) real\n", (int)fp);
         return;
     }
     /* try to reuse a slot */
@@ -438,8 +476,8 @@ file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
     if( control == IOBUFCTRL_UNDERFLOW ) {
 	assert( size ); /* need a buffer */
 	if ( a->eof_seen) {
-	    rc = -1;		
-	    *ret_len = 0;	
+	    rc = -1;
+	    *ret_len = 0;
 	}
 	else {
 #ifdef HAVE_DOSISH_SYSTEM
@@ -573,8 +611,8 @@ sock_filter (void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
     if( control == IOBUFCTRL_UNDERFLOW ) {
 	assert( size ); /* need a buffer */
 	if ( a->eof_seen) {
-	    rc = -1;		
-	    *ret_len = 0;	
+	    rc = -1;
+	    *ret_len = 0;
 	}
 	else {
             int nread;
@@ -883,7 +921,7 @@ print_chain( IOBUF a )
 						(byte*)&desc, &dummy_len );
 
 	log_debug("iobuf chain: %d.%d `%s' filter_eof=%d start=%d len=%d\n",
-		   a->no, a->subno, desc, a->filter_eof,
+		   a->no, a->subno, desc?desc:"?", a->filter_eof,
 		   (int)a->d.start, (int)a->d.len );
     }
 }
@@ -938,7 +976,8 @@ iobuf_close ( IOBUF a )
 	    log_error("iobuf_flush failed on close: %s\n", g10_errstr(rc));
 
 	if( DBG_IOBUF )
-	    log_debug("iobuf-%d.%d: close `%s'\n", a->no, a->subno, a->desc );
+	    log_debug("iobuf-%d.%d: close `%s'\n", a->no, a->subno,
+                      a->desc?a->desc:"?");
 	if( a->filter && (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE,
 					 a->chain, NULL, &dummy_len)) )
 	    log_error("IOBUFCTRL_FREE failed on close: %s\n", g10_errstr(rc) );
@@ -1042,7 +1081,7 @@ check_special_filename ( const char *fname )
         fname += 2;
         for (i=0; digitp (fname+i); i++ )
             ;
-        if ( !fname[i] ) 
+        if ( !fname[i] )
             return atoi (fname);
     }
     return -1;
@@ -1068,7 +1107,7 @@ iobuf_open( const char *fname )
     IOBUF a;
     FILEP_OR_FD fp;
     file_filter_ctx_t *fcx;
-    size_t len;
+    size_t len = 0;
     int print_only = 0;
     int fd;
 
@@ -1155,7 +1194,7 @@ iobuf_sockopen ( int fd, const char *mode )
     sock_filter( scx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
 	log_debug("iobuf-%d.%d: sockopen `%s'\n", a->no, a->subno, scx->fname);
-    iobuf_ioctl (a,3,1,NULL); /* disable fd caching */ 
+    iobuf_ioctl (a,3,1,NULL); /* disable fd caching */
 #else
     a = iobuf_fdopen (fd, mode);
 #endif
@@ -1199,7 +1238,8 @@ iobuf_create( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: create `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: create `%s'\n", a->no, a->subno,
+                  a->desc?a->desc:"?" );
 
     return a;
 }
@@ -1232,7 +1272,8 @@ iobuf_append( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: append `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: append `%s'\n", a->no, a->subno,
+                  a->desc?a->desc:"?" );
 
     return a;
 }
@@ -1260,7 +1301,8 @@ iobuf_openrw( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: openrw `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: openrw `%s'\n", a->no, a->subno,
+                  a->desc?a->desc:"?");
 
     return a;
 }
@@ -1272,7 +1314,8 @@ iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
     if ( cmd == 1 ) {  /* keep system filepointer/descriptor open */
         if( DBG_IOBUF )
             log_debug("iobuf-%d.%d: ioctl `%s' keep=%d\n",
-                      a? a->no:-1, a?a->subno:-1, a?a->desc:"?", intval );
+                      a? a->no:-1, a?a->subno:-1,
+                      a&&a->desc?a->desc:"?", intval );
         for( ; a; a = a->chain )
             if( !a->chain && a->filter == file_filter ) {
                 file_filter_ctx_t *b = a->filter_ov;
@@ -1293,15 +1336,16 @@ iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
                       ptrval? (char*)ptrval:"[all]");
         if ( !a && !intval ) {
 #ifndef FILE_FILTER_USES_STDIO
-            fd_cache_invalidate (ptrval);
+	    return fd_cache_invalidate (ptrval);
 #endif
             return 0;
-        }
+	 }
     }
     else if ( cmd == 3 ) {  /* disallow/allow caching */
         if( DBG_IOBUF )
             log_debug("iobuf-%d.%d: ioctl `%s' no_cache=%d\n",
-                      a? a->no:-1, a?a->subno:-1, a?a->desc:"?", intval );
+                      a? a->no:-1, a?a->subno:-1,
+                      a&&a->desc?a->desc:"?", intval );
         for( ; a; a = a->chain )
             if( !a->chain && a->filter == file_filter ) {
                 file_filter_ctx_t *b = a->filter_ov;
@@ -1316,6 +1360,23 @@ iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
             }
 #endif
     }
+    else if(cmd==4)
+      {
+	/* Do a fsync on the open fd and return any errors to the
+	   caller of iobuf_ioctl */
+        if( DBG_IOBUF )
+	  log_debug("iobuf-*.*: ioctl `%s' fsync\n",
+                      ptrval? (char*)ptrval:"<null>");
+
+	if(!a && !intval && ptrval)
+	  {
+#ifndef FILE_FILTER_USES_STDIO
+	    return fd_cache_synchronize (ptrval);
+#else
+	    return 0;
+#endif
+	  }
+      }
 
     return -1;
 }
@@ -1347,6 +1408,12 @@ iobuf_push_filter2( IOBUF a,
 
     if( a->use == 2 && (rc=iobuf_flush(a)) )
 	return rc;
+
+    if (a->subno >= MAX_NESTING_FILTER) {
+        log_error ("i/o filter too deeply nested - corrupted data?\n");
+        return G10ERR_UNEXPECTED;
+    }
+
     /* make a copy of the current stream, so that
      * A is the new stream and B the original one.
      * The contents of the buffers are transferred to the
@@ -1393,7 +1460,8 @@ iobuf_push_filter2( IOBUF a,
     f( ov, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &dummy_len );
 
     if( DBG_IOBUF ) {
-	log_debug("iobuf-%d.%d: push `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: push `%s'\n", a->no, a->subno,
+                  a->desc?a->desc:"?" );
 	print_chain( a );
     }
 
@@ -1419,7 +1487,8 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 	BUG();
 
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: pop `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: pop `%s'\n", a->no, a->subno,
+                  a->desc?a->desc:"?" );
     if( !a->filter ) { /* this is simple */
 	b = a->chain;
 	assert(b);
@@ -1497,7 +1566,7 @@ underflow(IOBUF a)
 	    IOBUF b = a->chain;
 	    if( DBG_IOBUF )
 		log_debug("iobuf-%d.%d: pop `%s' in underflow\n",
-					a->no, a->subno, a->desc );
+                          a->no, a->subno, a->desc?a->desc:"?" );
 	    xfree(a->d.buf);
 	    xfree(a->real_fname);
 	    memcpy(a, b, sizeof *a);
@@ -1562,8 +1631,8 @@ underflow(IOBUF a)
 	    if( !len && a->chain ) {
 		IOBUF b = a->chain;
 		if( DBG_IOBUF )
-		    log_debug("iobuf-%d.%d: pop `%s' in underflow (!len)\n",
-					       a->no, a->subno, a->desc );
+		    log_debug("iobuf-%d.%d: pop in underflow (!len)\n",
+                              a->no, a->subno);
 		xfree(a->d.buf);
 		xfree(a->real_fname);
 		memcpy(a,b, sizeof *a);
@@ -1863,13 +1932,24 @@ iobuf_get_filelength (IOBUF a, int *overflow )
     if (overflow)
       *overflow = 0;
 
-    if( a->directfp )  {
+    if (a->directfp)
+      {
 	FILE *fp = a->directfp;
 
-       if( !fstat(fileno(fp), &st) )
+#ifdef __VMS
+        /* 2009-02-19 SMS.
+         * On VMS, use a VMS-specific method to determine file size.
+         * For some non-UNIX-like file formats, the fstat() result
+         * will not agree with the C Standard I/O functions such as
+         * getc() and fread(), so these must be detected and handled
+         * specially. */
+        return vms_file_size (fileno( fp));
+#else /*!__VMS */
+        if( !fstat(fileno(fp), &st) )
            return st.st_size;
 	log_error("fstat() failed: %s\n", strerror(errno) );
 	return 0;
+#endif /*!__VMS */
     }
 
     /* Hmmm: file_filter may have already been removed */
@@ -1880,14 +1960,14 @@ iobuf_get_filelength (IOBUF a, int *overflow )
 
 #if defined(HAVE_DOSISH_SYSTEM) && !defined(FILE_FILTER_USES_STDIO)
             ulong size;
-            static int (* __stdcall get_file_size_ex) 
+            static int (* __stdcall get_file_size_ex)
               (void *handle, LARGE_INTEGER *size);
             static int get_file_size_ex_initialized;
 
             if (!get_file_size_ex_initialized)
               {
                 void *handle;
-                
+
                 handle = dlopen ("kernel32.dll", RTLD_LAZY);
                 if (handle)
                   {
@@ -1905,14 +1985,14 @@ iobuf_get_filelength (IOBUF a, int *overflow )
                    return a proper error in case a file is larger than
                    4GB. */
                 LARGE_INTEGER size;
-                
+
                 if (get_file_size_ex (fp, &size))
                   {
                     if (!size.u.HighPart)
                       return size.u.LowPart;
                     if (overflow)
                       *overflow = 1;
-                    return 0; 
+                    return 0;
                   }
               }
             else
@@ -1922,6 +2002,8 @@ iobuf_get_filelength (IOBUF a, int *overflow )
               }
             log_error ("GetFileSize for handle %p failed: %s\n",
                        fp, w32_strerror (0));
+#elif defined(__VMS)
+            return vms_file_size (my_fileno (fp));
 #else
             if( !fstat(my_fileno(fp), &st) )
 		return st.st_size;
@@ -1936,7 +2018,7 @@ iobuf_get_filelength (IOBUF a, int *overflow )
 
 /* Return the file descriptor of the underlying file or -1 if it is
    not available.  */
-int 
+int
 iobuf_get_fd (IOBUF a)
 {
   if (a->directfp)
@@ -2189,7 +2271,7 @@ iobuf_translate_file_handle ( int fd, int for_write )
 #ifdef _WIN32
     {
         int x;
-            
+
         if  ( fd <= 2 )
             return fd; /* do not do this for error, stdin, stdout, stderr */
 
@@ -2210,17 +2292,17 @@ static int
 translate_file_handle ( int fd, int for_write )
 {
 #ifdef _WIN32
-#ifdef FILE_FILTER_USES_STDIO  
+#ifdef FILE_FILTER_USES_STDIO
     fd = iobuf_translate_file_handle (fd, for_write);
 #else
     {
         int x;
 
-        if  ( fd == 0 ) 
+        if  ( fd == 0 )
             x = (int)GetStdHandle (STD_INPUT_HANDLE);
-        else if (fd == 1)    
+        else if (fd == 1)
             x = (int)GetStdHandle (STD_OUTPUT_HANDLE);
-        else if (fd == 2)    
+        else if (fd == 2)
             x = (int)GetStdHandle (STD_ERROR_HANDLE);
         else
             x = fd;
